from datetime import date
from enum import Enum
from typing import List, Optional

from kpler.sdk import Platform, SupplyDemandUsBalancesCrude, SupplyDemandUsBalancesGasoline
from kpler.sdk.client import KplerClient
from kpler.sdk.configuration import Configuration
from kpler.sdk.helpers import (
    process_date_parameter,
    process_enum_parameter,
    process_enum_parameters,
    process_list_parameter,
)


class SupplyDemandUsBalancesMetric:
    crude = SupplyDemandUsBalancesCrude
    gasoline = SupplyDemandUsBalancesGasoline


class SupplyDemand(KplerClient):
    """
    The ``SupplyDemand`` endpoint returns supply and demand metrics data values aggregated or split by country for a given product/ zone/ time period/selected snapshot,
    It also allows listing the available products and snapshots for the main endpoint
    """

    RESOURCE_NAME = "supply-demand"

    AVAILABLE_PLATFORMS = [Platform.Liquids]

    def __init__(self, configuration: Configuration, column_ids: bool = True, log_level=None):
        super().__init__(configuration, self.AVAILABLE_PLATFORMS, column_ids, log_level)

    def get(
        self,
        product: Optional[str] = None,
        metrics: Optional[List[Enum]] = None,
        split: Optional[Enum] = None,
        zones: Optional[List[str]] = None,
        start_date: Optional[date] = None,
        end_date: Optional[date] = None,
        snapshot_date: Optional[date] = None,
    ):
        """
        Args:
            product:  Optional[str] Name of product to retrieve data for
            metrics: Optional[List[Enum]] ``SupplyDemandMetric`` Returns only provided metrics
            split: Optional[Enum] ``SupplyDemandSplit`` Choose whether returned data are aggregated or split by country
            zones: Optional[List[str]] Names of countries/geographical zones
            start_date:  Optional[date] Start of the period (YYYY-MM-DD), must be after 2017-01-01
            end_date:  Optional[date] End of the period (YYYY-MM-DD), maximum of 18 month from today
            snapshot_date:  Optional[date] Date of the snapshot to retrieve data from (YYYY-MM-DD)

        Examples:
            >>> from datetime import date
            ... from kpler.sdk.resources.supply_demand import SupplyDemand
            ... from kpler.sdk import SupplyDemandSplit, SupplyDemandMetric
            ... sd_client = SupplyDemand(config)
            ... sd_client.get(
            ...     product="Crude/Co",
            ...     metrics=[SupplyDemandMetric.Supply, SupplyDemandMetric.Demand],
            ...     start_date=date(2020,10,1),
            ...     end_date=date(2020,11,1),
            ...     zones=["Japan"],
            ...     split=SupplyDemandSplit.Total
            ... )

            .. csv-table::
                :header: "Snapshot Date","Date","Product","Metric","Zones","Value","Unit"

                "2022-09-15","2020-10-01","Crude/Co",Supply (kbd),"Japan","3","kbd"
                "2022-09-15","2020-10-01","Crude/Co",Demand (kbd),"Japan","2279","kbd"
                "2022-09-15","2020-10-01","Crude/Co",Supply (kbd),"Japan","3","kbd"
                "2022-09-15","2020-10-01","Crude/Co",Demand (kbd),"Japan","2504","kbd"
        """

        query_parameters = {
            "product": product,
            "metrics": process_enum_parameters(metrics, to_lower_case=False),
            "zones": process_list_parameter(zones),
            "split": process_enum_parameter(split, to_lower_case=False),
            "startDate": process_date_parameter(start_date),
            "endDate": process_date_parameter(end_date),
            "snapshotDate": process_date_parameter(snapshot_date),
        }
        return self._get_dataframe(self.RESOURCE_NAME, query_parameters)

    def get_snapshots(
        self,
        product: Optional[str] = None,
        start_date: Optional[date] = None,
        end_date: Optional[date] = None,
    ):
        """
        Args:
            product:  Optional[str] Name of product to retrieve snapshots for
            start_date:  Optional[date] Start of the period (YYYY-MM-DD)
            end_date:  Optional[date] End of the period (YYYY-MM-DD)
        Examples:
            >>> from datetime import date
            ... from kpler.sdk.resources.supply_demand import SupplyDemand
            ... sd_client = SupplyDemand(config)
            ... sd_client.get_snapshots(
            ...     product="Crude/Co",
            ...     start_date=date(2022,08,1),
            ...     end_date=date(2022,10,1),
            ... )

            .. csv-table::
                :header: "Date","Product"

                "2022-08-14","Crude/Co"
                "2022-09-01","Crude/Co"
        """
        query_parameters = {
            "product": product,
            "startDate": process_date_parameter(start_date),
            "endDate": process_date_parameter(end_date),
        }
        return self._get_dataframe(self.RESOURCE_NAME + "/snapshots", query_parameters)

    def get_products(self):
        """
        Args: None
        Examples:
            >>> from datetime import date
            ... from kpler.sdk.resources.supply_demand import SupplyDemand
            ... sd_client = SupplyDemand(config)
            ... sd_client.get_products()

            .. csv-table::
                :header: "Product","metrics","Snapshot Start Date","Snapshot End Date"

                Crude/Co,"supply,demand,refineryRun,directCrudeUse,balance,netExport,stockChange,balancingFactor",2022-08-15,2022-09-15
        """
        return self._get_dataframe(self.RESOURCE_NAME + "/products", params={})

    def get_us_balances(
        self,
        product: Optional[Enum] = None,
        metrics: Optional[List[Enum]] = None,
        start_date: Optional[date] = None,
        end_date: Optional[date] = None,
        granularity: Optional[Enum] = None,
    ):
        """
        Args:
            product:  Optional[Enum] Name of product to retrieve data for
            metrics: Optional[List[Enum]] ``SupplyDemandMetric`` Returns only provided metrics
            start_date:  Optional[date] Start of the period (YYYY-MM-DD), must be after 2017-01-01
            end_date:  Optional[date] End of the period (YYYY-MM-DD), maximum of 12 month from today
            granularity:  Optional[Enum] ``SupplyDemandUsBalancesGranularity`` Choose the granularity of the data

        Examples:
            >>> from datetime import date
            ... from kpler.sdk.resources.supply_demand import SupplyDemand, SupplyDemandUsBalancesMetric
            ... from kpler.sdk import SupplyDemandUsBalancesGranularity, SupplyDemandUsBalancesProduct
            ... sd_client = SupplyDemand(config)
            ... sd_client.get_us_balances(
            ...     product=SupplyDemandUsBalancesProduct.Crude,
            ...     metrics=[SupplyDemandUsBalancesMetric.crude.NetImport],
            ...     start_date=date(2020,1,1),
            ...     end_date=date(2020,6,1),
            ...     granularity=SupplyDemandUsBalancesGranularity.Weekly

            .. csv-table::
                :header: "Snapshot Date","Date","Product","Metric","Zones","Value","Unit"

                "2024-03-28","2023-01-01","Crude/Co","netImport","United States","3243","kbd"
                "2024-03-28","2023-02-01","Crude/Co","netImport","United States","2657","kbd"
                "2024-03-28","2023-03-01","Crude/Co","netImport","United States","2261","kbd"
        """
        query_parameters = {
            "product": process_enum_parameter(product, to_lower_case=False),
            "metrics": process_enum_parameters(metrics, to_lower_case=False),
            "startDate": process_date_parameter(start_date),
            "endDate": process_date_parameter(end_date),
            "granularity": process_enum_parameter(granularity, to_lower_case=False),
        }
        return self._get_dataframe(self.RESOURCE_NAME + "/us-balances", query_parameters)



from datetime import date
from enum import Enum
from typing import List, Optional

from pandas import DataFrame

from kpler.sdk import Platform
from kpler.sdk.client import KplerClient
from kpler.sdk.configuration import Configuration
from kpler.sdk.helpers import (
    process_bool_parameter,
    process_date_parameter,
    process_enum_parameter,
    process_enum_parameters,
    process_list_parameter,
)


class CongestionVessels(KplerClient):

    """
    The ``CongestionVessels`` endpoint returns current and historical waiting time for vessels spent in waiting areas before going to berth.
    """

    RESOURCE_NAME = "congestion/vessels"
    COLUMNS_RESOURCE_NAME = "congestion"

    AVAILABLE_PLATFORMS = [Platform.Dry, Platform.Liquids, Platform.LNG, Platform.LPG]

    def __init__(self, configuration: Configuration, column_ids: bool = True, log_level=None):
        super().__init__(configuration, self.AVAILABLE_PLATFORMS, column_ids, log_level)

    def get_columns(self) -> DataFrame:
        """
        This endpoint returns a recent and updated list of all columns available for the endpoint congestion vessels.

        Examples:
            >>> from kpler.sdk.resources.congestion_vessels import CongestionVessels
            ... congestion_vessels_client=CongestionVessels(config)
            ... congestion_vessels_client.get_columns()

            .. csv-table::
                :header: "id","name","description","deprecated","type"

                "date","Date","Day of vessel in the waiting zone","False","date yyyy-MM-dd"
                "vessel_imo","IMO","IMO of the vessel","False","long"
                "vessel","Name","Name of the vessel","False","string"
                "vessel_dwt_tons","Dead Weight Tonnage","Dead Weight Tonnage","False","long"
                "cargo_tons","Cargo (t)","Cargo in tons","False","double"
                "...","...","...","...","..."
        """
        return self._get_columns_for_resource(self.COLUMNS_RESOURCE_NAME)

    def get(
        self,
        start_date: Optional[date] = None,
        end_date: Optional[date] = None,
        zones: Optional[List[str]] = None,
        products: Optional[List[str]] = None,
        vessel_types_cpp: Optional[List[Enum]] = None,
        vessel_types_oil: Optional[List[Enum]] = None,
        size: Optional[int] = None,
        vessel_types: Optional[List[Enum]] = None,
        gte: Optional[int] = None,
        lte: Optional[int] = None,
        congestion_only: bool = None,
        vessel_operation: Optional[str] = None,
        waiting_duration_min: Optional[int] = None,
        waiting_duration_max: Optional[int] = None,
        with_freight_view: bool = False,
        period: Optional[Enum] = None,
        vessels: Optional[List[str]] = None,
    ):

        """
        Args:
            start_date: Optional[date] Start of the period (YYYY-MM-DD)
            end_date: Optional[date] End of the period (YYYY-MM-DD)
            zones: Optional[List[str]] Names of countries/geographical zones
            products: Optional[List[str]] Names of products
            vessel_types_cpp: Optional[List[Enum]] ``VesselsTypesCpp``
            vessel_types_oil: Optional[List[Enum]] ``VesselsTypesOil``
            size: Optional[int] Maximum number of fleet utilization vessels returned
            vessel_types: Optional[List[Enum]] ``VesselsTypesDry`` ``VesselsTypesLNG`` ``VesselsTypesLPG``
            gte: Optional[int] Get vessels with deadweight/capacity greater or equals to this value by default 0
            lte: Optional[int] Get vessels with deadweight/capacity lower or equals to this value by default 606550
            congestion_only: bool Exclude vessels in the waiting zone not tagged as congestion (lays ups,open vessels,floating storage,distressed cargoes etc).Use ["true", "false"]
            vessel_operation: ``CongestionVesselsOperation``
            waiting_duration_min: Optional[int] Minimum waiting duration of vessel. Default to 0 days if not specified.
            waiting_duration_max: Optional[int] Maximum waiting duration of vessel. Default to 1000000 days if not specified.
            with_freight_view: bool By default: with_freight_view=False. Provides access to the entire fleet's trades, irrespective of your current cargo subscription. Only available via Freight subscription.
            period: Optional[Enum] ``CongestionVesselsPeriod``
            vessels: Optional[List[str]] Names or IMOs of the vessels

        Examples:
            >>> from datetime import date
            ... from kpler.sdk.resources.congestion_vessels import CongestionVessels
            ... congestion_vessels_client=CongestionVessels(config)
            ... congestion_vessels_client.get(
            ...         start_date=date(2020, 10, 1),
            ...         end_date=date(2020, 11, 1),
            ...         zones=["Japan"],
            ...         products=["gasoline", "DPP"]
            ...     )

            .. csv-table::
                :header:  "Date (timestamp)","IMO","Name","Dead Weight Tonnage","Cargo(t)","Family","Group","Products","Installation","Port","..."

                "2020-10-25","9392808","Alpine Mystery","49999","16521.72","DPP","NaN","NaN","Kawasaki Refinery","Kawasaki","..."
                "2020-10-11","9317078","Haruna Express","45761","32449.03","Light Ends","Gasoline/Naphtha","Gasoline","Showa Yokkaichi Refinery","Yokkaichi","Japan","..."
                "2020-10-31","9392808","Alpine Mystery","49999","21028.51","DPP","Fuel Oils","FO","Sakai Refinery","Osaka","..."
                "2020-10-26","9392808","Alpine Mystery","49999","16521.72","DPP","NaN","NaN","Kawasaki Refinery","Kawasaki","..."
                "2020-10-28","9290646","Torm Kansas","46922","16825.78","Light Ends","Gasoline/Naphtha","Gasoline","Showa Yokkaichi Refinery","Yokkaichi","..."
        """

        query_parameters = {
            "startDate": process_date_parameter(start_date),
            "endDate": process_date_parameter(end_date),
            "zones": process_list_parameter(zones),
            "products": process_list_parameter(products),
            "vesselTypesCpp": process_enum_parameters(vessel_types_cpp, False),
            "vesselTypesOil": process_enum_parameters(vessel_types_oil, False),
            "size": size,
            "vesselTypes": process_enum_parameters(vessel_types, False),
            "gte": gte,
            "lte": lte,
            "congestionOnly": process_bool_parameter(congestion_only),
            "vesselOperation": process_enum_parameter(vessel_operation),
            "waitingDurationMin": waiting_duration_min,
            "waitingDurationMax": waiting_duration_max,
            "withFreightView": process_bool_parameter(with_freight_view),
            "period": process_enum_parameter(period),
            "vessels": process_list_parameter(vessels),
        }
        return self._get_dataframe(self.RESOURCE_NAME, query_parameters)



from datetime import date
from enum import Enum
from typing import List, Optional

from kpler.sdk import Platform
from kpler.sdk.client import KplerClient
from kpler.sdk.configuration import Configuration
from kpler.sdk.helpers import (
    process_bool_parameter,
    process_date_parameter,
    process_enum_parameter,
    process_enum_parameters,
    process_list_parameter,
)


class CongestionSeries(KplerClient):

    """
    The ``CongestionSeries`` endpoint returns current and historical waiting time for vessels spent in waiting areas before going to berth.
    """

    RESOURCE_NAME = "congestion/series"

    AVAILABLE_PLATFORMS = [Platform.Dry, Platform.Liquids, Platform.LNG, Platform.LPG]

    def __init__(self, configuration: Configuration, column_ids: bool = True, log_level=None):
        super().__init__(configuration, self.AVAILABLE_PLATFORMS, column_ids, log_level)

    def get(
        self,
        start_date: Optional[date] = None,
        end_date: Optional[date] = None,
        zones: Optional[List[str]] = None,
        metric: Optional[Enum] = None,
        period: Optional[Enum] = None,
        split: Optional[Enum] = None,
        products: Optional[List[str]] = None,
        unit: Optional[Enum] = None,
        vessel_types_cpp: Optional[List[Enum]] = None,
        vessel_types_oil: Optional[List[Enum]] = None,
        vessel_types: Optional[List[Enum]] = None,
        gte: Optional[int] = None,
        lte: Optional[int] = None,
        congestion_only: bool = None,
        vessel_operation: Optional[Enum] = None,
        waiting_duration_min: Optional[int] = None,
        waiting_duration_max: Optional[int] = None,
        with_freight_view: bool = False,
    ):

        """
        Args:
            start_date: Optional[date] Start of the period (YYYY-MM-DD)
            end_date: Optional[date] End of the period (YYYY-MM-DD)
            zones: Optional[List[str]] Names of countries/geographical zones
            metric: Optional[Enum] ``CongestionSeriesMetric``
            period: Optional[Enum] ``CongestionSeriesPeriod``
            split: Optional[Enum] ``CongestionSeriesSplit``
            products: Optional[List[str]] Names of products
            unit: Optional[Enum] ``CongestionSeriesUnit``
            vessel_types_cpp: Optional[List[Enum]] ``VesselTypesCPP``
            vessel_types_oil: Optional[List[Enum]] ``VesselTypesOil``
            vessel_types: Optional[List[Enum]] ``VesselTypesDry`` ``VesselTypesLNG`` ``VesselTypesLPG``
            gte: Optional[int] Get vessels with deadweight/capacity greater or equals to this value by default 0
            lte: Optional[int] Get vessels with deadweight/capacity lower or equals to this value by default 606550
            congestion_only: bool Exclude vessels in the waiting zone not tagged as congestion (lays ups,open vessels,floating storage,distressed cargoes etc).Use ["true", "false"]
            vessel_operation: Optional[Enum] ``CongestionSeriesOperation``
            waiting_duration_min: Optional[int] Minimum waiting duration of vessel. Default to 0 days if not specified.
            waiting_duration_max: Optional[int] Maximum waiting duration of vessel. Default to 1000000 days if not specified.
            with_freight_view: bool By default: with_freight_view=False. Provides access to the entire fleet's trades, irrespective of your current cargo subscription. Only available via Freight subscription.

        Examples:
            >>> from datetime import date
            ... from kpler.sdk.resources.congestion_series import CongestionSeries
            ... from kpler.sdk import CongestionSeriesMetric, CongestionSeriesPeriod, CongestionSeriesSplit
            ... congestion_series_client = CongestionSeries(config)
            ... congestion_series_client.get(
            ...     start_date=date(2020, 1, 1),
            ...     end_date=date(2020, 6, 1),
            ...     metric=CongestionSeriesMetric.Count,
            ...     period=CongestionSeriesPeriod.Monthly,
            ...     split=CongestionSeriesSplit.VesselType
            ... )

            .. csv-table::
                :header:  "Date","MR","GP","LR3","LR2","VLCC","LR1"

                "2020-01","6097","5636","742","2241","866","1022"
                "2020-02","5683","5232","685","2094","831","944"
                "2020-03","6145","5659","747","2223","905","1033"
                "2020-04","5467","8006","676","2073","906","887"
                "2020-05","5414","9446","650","2045","855","838"
                "2020-06","170","268","21","72","22","22"
        """

        query_parameters = {
            "startDate": process_date_parameter(start_date),
            "endDate": process_date_parameter(end_date),
            "zones": process_list_parameter(zones),
            "metric": process_enum_parameter(metric),
            "period": process_enum_parameter(period),
            "split": process_enum_parameter(split),
            "products": process_list_parameter(products),
            "unit": process_enum_parameter(unit),
            "vesselTypesCpp": process_enum_parameters(vessel_types_cpp, False),
            "vesselTypesOil": process_enum_parameters(vessel_types_oil, False),
            "vesselTypes": process_enum_parameters(vessel_types, False),
            "gte": gte,
            "lte": lte,
            "congestionOnly": process_bool_parameter(congestion_only),
            "vesselOperation": process_enum_parameter(vessel_operation, False),
            "waitingDurationMin": waiting_duration_min,
            "waitingDurationMax": waiting_duration_max,
            "withFreightView": process_bool_parameter(with_freight_view),
        }
        return self._get_dataframe(self.RESOURCE_NAME, query_parameters)



from datetime import date
from enum import Enum
from typing import List, Optional

from pandas import DataFrame

from kpler.sdk import Platform
from kpler.sdk.client import KplerClient
from kpler.sdk.configuration import Configuration
from kpler.sdk.helpers import (
    process_date_parameter,
    process_enum_parameter,
    process_enum_parameters,
    process_list_parameter,
)


class CrudeImports(KplerClient):
    """
    The CrudeImports endpoint returns the 30days moving average crude imports
    for a point of interest (installation/zone) on a daily, weekly, weekly EIA (for US), monthly and yearly basis.
    It also provides the weighted average API and Sulfur values over time.
    """
    API_SULFUR_RESOURCE_NAME = "refineries/api-sulfur"

    RESOURCE_NAME = "refineries/crude-imports"

    AVAILABLE_PLATFORMS = [Platform.Liquids]

    def __init__(self, configuration: Configuration, column_ids: bool = True, log_level=None):
        super().__init__(configuration, self.AVAILABLE_PLATFORMS, column_ids, log_level)

    def get_columns(self) -> DataFrame:
        """
        This endpoint returns a recent and updated list of all columns available for the endpoint
        refineries crude-imports.


        Examples:
            >>> from kpler.sdk.resources.refineries.crude_imports import CrudeImports
            ... crude_imports_client = CrudeImports(config)
            ... crude_imports_client.get_columns()

            .. csv-table::
                :header: "id","name","description","deprecated","type"

                "date","Date (timestamp)","Date (timestamp)","False","date yyyy-MM-dd"
                "zones","Zones","Zones the row's data uses","False","list of string"
                "installations","Installations","Instalations the row's data uses","False","list of string"
                "splitValue","Split Value","Split Value","False","string"
                "metric","Metric","Metric","False","string"
                "...","..."
        """
        return self._get_columns_for_resource(self.RESOURCE_NAME)

    def get_api_sulfur_columns(self) -> DataFrame:
        """
        This endpoint returns a recent and updated list of all columns available for the endpoint refineries
        crude-imports api-sulfur.


        Examples:
            >>> from kpler.sdk.resources.refineries.crude_imports import CrudeImports
            ... crude_imports_client = CrudeImports(config)
            ... crude_imports_client.get_api_sulfur_columns()

            .. csv-table::
                :header: "id","name","description","deprecated","type"

                "date","Date (timestamp)","Date, within the start_date and end_date. Data is provided with ascending order on date. Format YYYY-MM-DD.","False","string"
                "zones","Zones","List of zones specified in the parameter zones.","False","list of string"
                "installations","Installations","List of installations specified in the parameter installations.","False","list of string"
                "splitValue","Split Value","Name of the bucket corresponding to the specified split. EG Americas or Asia for split by Continent.","False","string"
                "metric","Metric","Corresponding to the endpoint.","False","string"
                "...","..."
        """
        return self._get_columns_for_resource(self.API_SULFUR_RESOURCE_NAME)

    def get(
            self,
            players: Optional[List[str]] = None,
            installations: Optional[List[str]] = None,
            zones: Optional[List[str]] = None,
            start_date: Optional[date] = None,
            end_date: Optional[date] = None,
            products: Optional[List[Enum]] = None,
            unit: Optional[Enum] = None,
            granularity: Optional[Enum] = None,
            split: Optional[Enum] = None,
            metric: Optional[Enum] = None

    ) -> DataFrame:
        """
        Args:
            players:  Optional[str] Names of players
            installations: Optional[List[str]] Names of installations
            zones: Optional[List[str]] Names of countries/geographical zones
            start_date:  Optional[date] Start of the period (YYYY-MM-DD), must be after 2017-01-01
            end_date:  Optional[date] End of the period (YYYY-MM-DD), maximum of 7 days from today
            products: Optional[List[Enum]] ``RunsQualities`` Names of qualities
            unit: Optional[Enum] ``ImportsUnit``
            granularity: Optional[Enum] ``ImportsGranularity``
            split: Optional[Enum] ``ImportsSplit``
            metric: Optional[Enum] ``ImportsMetric`` Returns only provided metrics
        Examples:
            >>> from datetime import date
            ... from kpler.sdk.resources.refineries.crude_imports import CrudeImports
            ... from kpler.sdk import ImportsSplit, ImportsGranularity, RunsQualities, ImportsUnit, ImportsMetric
            ... crude_imports_client = CrudeImports(config)
            ... crude_imports_client.get(
            ...     installations=["xxxx"],
            ...     zones=["United States"],
            ...     start_date=date(2023, 4, 1),
            ...     end_date=date(2023, 7, 31),
            ...     player=["xxx"],
            ...     products=[RunsQualities.All],
            ...     unit=ImportsUnit.KBD,
            ...     granularity=ImportsGranularity.Monthly,
            ...     split=ImportsSplit.Total,
            ...     metric=ImportsMetric.Quantity,
            ... )

            .. csv-table::
                :header: "Date","Zones","Installations","Split Value","Metric","Value","Unit"

                "2023-04-01","United States",,"Total","Imports","79644.0","kbd"
                "2023-05-01","United States",,"Total","Imports","79417.0","kbd"
                "2023-06-01","United States",,"Total","Imports","79540.0","kbd"
                "2023-07-01","United States",,"Total","Imports","81059.0","kbd"

                "2023-04-01","United States",,"Total","API - Weighted average","33.5","° API"
                "2023-04-01","United States",,"Total","Sulfur - Weighted average","1.24","% wt"
                "2023-05-01","United States",,"Total","API - Weighted average","33.4","° API"
                "2023-05-01","United States",,"Total","Sulfur - Weighted average","1.25","% wt"
                "2023-06-01","United States",,"Total","API - Weighted average","33.3","° API"
        """

        query_parameters = {
            "players": process_list_parameter(players),
            "installations": process_list_parameter(installations),
            "zones": process_list_parameter(zones),
            "startDate": process_date_parameter(start_date),
            "endDate": process_date_parameter(end_date),
            "products": process_enum_parameters(products, to_lower_case=False),
            "split": process_enum_parameter(split, to_lower_case=False),
            "granularity": process_enum_parameter(granularity, to_lower_case=False),
            "unit": process_enum_parameter(unit, to_lower_case=False),
            "metric": process_enum_parameter(metric, to_lower_case=False)
        }
        return self._get_dataframe(self.RESOURCE_NAME, query_parameters)



from datetime import date, timedelta

from kpler.sdk import Platform
from kpler.sdk.configuration import Configuration
from kpler.sdk.resources.trades import Trades


config = Configuration(Platform.Liquids, "<your email>", "<your password>")

trades_client = Trades(config)

# Get US imports over last week
us_imports = trades_client.get(
    to_zones=["United States"],
    products=["crude"],
    with_forecast=False,
    with_intra_country=True,
    start_date=date.today() - timedelta(days=7),
    columns=[
        "vessel_name",
        "closest_ancestor_product",
        "closest_ancestor_grade",
        "start",
        "end",
        "origin_location_name",
        "destination_location_name",
    ],
)



from datetime import date
from enum import Enum
from typing import List, Optional

from pandas import DataFrame

from kpler.sdk import Platform
from kpler.sdk.client import KplerClient
from kpler.sdk.configuration import Configuration
from kpler.sdk.helpers import (
    process_bool_parameter,
    process_date_parameter,
    process_enum_parameter,
    process_enum_parameters,
)


class FleetDevelopmentSeries(KplerClient):

    """
    The ``FleetDevelopmentSeries`` returns In-Service fleet, new vessel deliveries and old vessels sold for scrap as well as vessels with Scrubbers installed and planned.
    """

    RESOURCE_NAME = "fleet-development/series"
    RESOURCE_GET_COLUMN_NAME = "fleet-development"

    AVAILABLE_PLATFORMS = [Platform.Liquids, Platform.LPG, Platform.LNG, Platform.Dry]

    def __init__(self, configuration: Configuration, column_ids: bool = True, log_level=None):
        super().__init__(configuration, self.AVAILABLE_PLATFORMS, column_ids, log_level)

    def get_columns(self) -> DataFrame:
        """
        This endpoint returns a recent and updated list of all columns available for the fleet_development endpoints

        Examples:
            >>> from kpler.sdk.resources.fleet_development_series import FleetDevelopmentSeries
            ... fleet_development_series_client = FleetDevelopmentSeries(config)
            ... fleet_development_series_client.get_columns()

            .. csv-table::
                :header: "id","name","description","deprecated","type"

                "day","Day","None","False","date yyyy-MM-dd"
                "vessel_name","Name","Name of the vessel","False","string"
                "vessel_imo","IMO","Vessel IMO","False","string"
                "vessel_mmsi","MMSI","The Maritime Mobile Service Identity of the vessel (9 digits)","False","string"
                "vessel_status","Status","Status of the vessel","False","string"
                "...","...","...","...","..."
        """
        return self._get_columns_for_resource(self.RESOURCE_GET_COLUMN_NAME)

    def get(
        self,
        start_date: Optional[date] = None,
        end_date: Optional[date] = None,
        metric: Optional[Enum] = None,
        aggregation_metric: Optional[Enum] = None,
        period: Optional[Enum] = None,
        split: Optional[Enum] = None,
        unit: Optional[Enum] = None,
        vessel_types_cpp: Optional[List[Enum]] = None,
        vessel_types_oil: Optional[List[Enum]] = None,
        vessel_types: Optional[List[Enum]] = None,
        gte: Optional[int] = None,
        lte: Optional[int] = None,
        with_orderbook: bool = False,
    ):

        """
        Args:
            start_date: Optional[date] Start of the period (YYYY-MM-DD)
            end_date: Optional[date] End of the period (YYYY-MM-DD)
            metric: Optional[Enum] ``FleetDevelopmentSeriesMetric``
            aggregation_metric: Optional[Enum] ``FleetDevelopmentSeriesAggregationMetric``
            period: Optional[Enum] ``FleetDevelopmentSeriesPeriod``
            split: Optional[Enum] ``FleetDevelopmentSeriesSplit``
            unit: Optional[Enum] ``FleetDevelopmentSeriesUnit``
            vessel_types_cpp: Optional[List[Enum]] ``VesselTypesCpp``
            vessel_types_oil: Optional[List[Enum]] ``VesselTypesOil``
            vessel_types: Optional[List[Enum]] ``VesselTypesDry`` ``VesselTypesLNG`` ``VesselTypesLPG``
            gte: Optional[int] Get vessels with deadweight/capacity greater or equals to this value by default 0
            lte: Optional[int] Get vessels with deadweight/capacity lower or equals to this value by default 606550
            with_orderbook: bool = False Access vessels in fleet orderbook, scheduled for future delivery

        Examples:

            >>> from datetime import date
            ... from kpler.sdk.resources.fleet_development_series import FleetDevelopmentSeries
            ... from kpler.sdk import FleetDevelopmentSeriesMetric, FleetDevelopmentSeriesAggregationMetric, FleetDevelopmentSeriesPeriod, FleetDevelopmentSeriesSplit
            ... fleet_development_series_client = FleetDevelopmentSeries(config)
            ... fleet_development_series_client.get(
            ...         start_date=date(2020,1,1),
            ...         end_date=date(2020,5,1),
            ...         metric=FleetDevelopmentSeriesMetric.Available,
            ...         aggregation_metric=FleetDevelopmentSeriesAggregationMetric.Count,
            ...         period=FleetDevelopmentSeriesPeriod.Monthly,
            ...         split=FleetDevelopmentSeriesSplit.VesselType
            ... )

            .. csv-table::
                :header:  "Date","GP","MR","LR2","VLCC","LR3","LR1"

                "2020-01","3727","2325","1107","905","636","506"
                "2020-02","3725","2330","1106","906","639","506"
                "2020-03","3728","2340","1108","909","641","506"
                "2020-04","3737","2347","1109","911","642","507"
                "2020-05","3740","2347","1109","911","642","507"
        """

        query_parameters = {
            "startDate": process_date_parameter(start_date),
            "endDate": process_date_parameter(end_date),
            "metric": process_enum_parameter(metric),
            "aggregationMetric": process_enum_parameter(aggregation_metric),
            "period": process_enum_parameter(period),
            "split": process_enum_parameter(split),
            "unit": process_enum_parameter(unit),
            "vesselTypesCpp": process_enum_parameters(vessel_types_cpp, False),
            "vesselTypesOil": process_enum_parameters(vessel_types_oil, False),
            "vesselTypes": process_enum_parameters(vessel_types, False),
            "gte": gte,
            "lte": lte,
            "withOrderbook": process_bool_parameter(with_orderbook),
        }
        return self._get_dataframe(self.RESOURCE_NAME, query_parameters)



from datetime import date
from enum import Enum
from typing import List, Optional

from pandas import DataFrame

from kpler.sdk import Platform
from kpler.sdk.client import KplerClient
from kpler.sdk.configuration import Configuration
from kpler.sdk.helpers import (
    process_bool_parameter,
    process_date_parameter,
    process_enum_parameter,
    process_list_parameter,
)


class Inventories(KplerClient):

    """
    The ``Inventories`` endpoint returns inland crude oil stocks of a given point of interest, with a specific periodicity over a period of time.
    Waterborne Cargoes, local Supply and Demand will also be returned for the given point of interest.
    """

    RESOURCE_NAME = "inventories"

    AVAILABLE_PLATFORMS = [Platform.Liquids]

    def __init__(self, configuration: Configuration, column_ids: bool = True, log_level=None):
        super().__init__(configuration, self.AVAILABLE_PLATFORMS, column_ids, log_level)

    def get_columns(self) -> DataFrame:
        """
        This endpoint returns a recent and updated list of all columns available for the endpoint inventories.

        Examples:
            >>> from kpler.sdk.resources.inventories import Inventories
            ... inventories_client = Inventories(config)
            ... inventories_client.get_columns()

            .. csv-table::
                :header: "id","name","description","deprecated","type"

                "date","Date","Date for which values are provided","False","date yyyy-MM-dd"
                "zone_name","Zone","Name of the area of interest","False","string"
                "installation_name","Installation","Name of the installation","False","string"
                "level_barrels","Level (kb)","Amount of crude oil stored in kb","False","double"
                "local_suply_barrels","Local Supply (kbd)","Net overland inflow of crude oil (due to local oilfields, or pipelines) in kbd","False","double"
                "...","...","...","...","..."
        """
        return self._get_columns_for_resource(self.RESOURCE_NAME)

    def get(
        self,
        size: Optional[int] = None,
        start_date: Optional[date] = None,
        end_date: Optional[date] = None,
        installations: Optional[List[str]] = None,
        zones: Optional[List[str]] = None,
        period: Optional[Enum] = None,
        split: Optional[Enum] = None,
        columns: Optional[List[str]] = None,
        cargo_tracking_enhancement: Optional[bool] = None,
        add_commodities_on_water: Optional[bool] = None,
        floating_storage_duration_min: Optional[str] = None,
        floating_storage_duration_max: Optional[str] = None,
        without_eia_adjustment: Optional[bool] = None,
    ):
        """
        Args:
            size:  Optional[int] Maximum number of contracts returned
            start_date:  Optional[date] Start of the period (YYYY-MM-DD)
            end_date:  Optional[date] End of the period (YYYY-MM-DD)
            installations: Optional[List[str]] Names of installations
            zones: Optional[List[str]] Names of countries/geographical zones
            period: Optional[Enum] ``InventoriesPeriod``
            split: Optional[Enum] ``InventoriesSplit``
            columns: Optional[List[str]] Retrieve all available columns when set to "all"
            cargo_tracking_enhancement:  Optional[bool] Use cargo tracking data to fill the blanks in between satellite images (default: False)
            add_commodities_on_water: Optional[bool] Provides total volume of inventories including offshore and offshore. Default to false if not specified.
            floating_storage_duration_min: Optional[str] Minimum floating days [7-10-12-15-20-30-90]
            floating_storage_duration_max: Optional[str] Maximum floating days [10-12-15-20-30-90-Inf]
            without_eia_adjustment:  Optional[bool] No use of EIA data for enhancing inventories volumes (default: False)

        Examples:
            >>> from datetime import date
            ... from kpler.sdk.resources.inventories import Inventories
            ... from kpler.sdk import InventoriesSplit
            ... inventories_client = Inventories(config)
            ... inventories_client.get(
            ...     start_date=date(2020,10,1),
            ...     end_date=date(2020,11,1),
            ...     zones=["Japan"],
            ...     split=InventoriesSplit.Total
            ... )

            .. csv-table::
                :header: "Date","Zone","Installation","Level (kb)","Local Supply (kbd)","Local Demand (kbd)","Cargoes (kbd)","Capacity (kb)","..."

                "2020-10-01 23:59:00","Japan","NaN","354342","0","2377","2594","474008"","
                "2020-10-02 23:59:00","Japan","NaN","354492","0","2090","2705","474008"
                "2020-10-03 23:59:00","Japan","NaN","354178","0","2087","1101","474008"
                "2020-10-04 23:59:00","Japan","NaN","354073","0","1883","2500","474008"
                "2020-10-05 23:59:00","Japan","NaN","354187","0","1862","916","474008"
                "...","...","...","...","...","...","...","..."

        """

        query_parameters = {
            "size": size,
            "startDate": process_date_parameter(start_date),
            "endDate": process_date_parameter(end_date),
            "installations": process_list_parameter(installations),
            "zones": process_list_parameter(zones),
            "period": process_enum_parameter(period),
            "split": process_enum_parameter(split),
            "columns": process_list_parameter(columns),
            "cargoTrackingEnhancement": process_bool_parameter(cargo_tracking_enhancement),
            "addCommoditiesOnWater": process_bool_parameter(add_commodities_on_water),
            "floatingStorageDurationMin": floating_storage_duration_min,
            "floatingStorageDurationMax": floating_storage_duration_max,
            "withoutEiaAdjustment": process_bool_parameter(without_eia_adjustment),
        }
        return self._get_dataframe(self.RESOURCE_NAME, query_parameters)



from datetime import date
from enum import Enum
from typing import List, Optional

from kpler.sdk import Platform
from kpler.sdk.client import KplerClient
from kpler.sdk.configuration import Configuration
from kpler.sdk.helpers import (
    process_bool_parameter,
    process_date_parameter,
    process_enum_parameters,
    process_flow_snapshot_parameter,
    process_list_parameter,
)


class Flows(KplerClient):
    """
    The ``Flows`` endpoint returns the aggregated flows for a point of interest (installation/zone) on a daily, weekly,
    weekly EIA (for US), monthly and yearly basis.
    """

    RESOURCE_NAME = "flows"

    AVAILABLE_PLATFORMS = [Platform.Dry, Platform.Liquids, Platform.LNG, Platform.LPG]

    def __init__(self, configuration: Configuration, column_ids: bool = True, log_level=None):
        super().__init__(configuration, self.AVAILABLE_PLATFORMS, column_ids, log_level)

    def get(
        self,
        flow_direction: Optional[List[Enum]] = None,
        split: Optional[List[Enum]] = None,
        granularity: Optional[List[Enum]] = None,
        start_date: Optional[date] = None,
        end_date: Optional[date] = None,
        from_installations: Optional[List[str]] = None,
        to_installations: Optional[List[str]] = None,
        from_zones: Optional[List[str]] = None,
        to_zones: Optional[List[str]] = None,
        products: Optional[List[str]] = None,
        only_realized: Optional[bool] = None,
        unit: Optional[List[Enum]] = None,
        with_intra_country: Optional[bool] = None,
        with_intra_region: Optional[bool] = None,
        with_forecast: Optional[bool] = None,
        with_freight_view: bool = False,
        vessel_types: Optional[List[Enum]] = None,
        vessel_types_alt: Optional[List[Enum]] = None,
        with_product_estimation: bool = False,
        snapshot_date: Optional[date] = None, 
    ):
        """

        Args:
             flow_direction: Optional[List[Enum]] ``FlowsDirection``
             split: Optional[List[Enum]] ``FlowsSplit``
             granularity: Optional[List[Enum]] ``FlowsPeriod``
             start_date: Optional[date] Start of the period (YYYY-MM-DD)
             end_date: Optional[date] End of the period (YYYY-MM-DD)
             from_installations: Optional[List[str]] Names of installations
             to_installations: Optional[List[str]] Names of installations
             from_zones: Optional[List[str]] Names of zones ["port", "region", "country", "continent"]
             to_zones: Optional[List[str]] = Names of zones ["port", "region", "country", "continent"]
             products: Optional[List[str]] = Names of products
             only_realized: Optional[bool] By default: onlyRealized=false. Takes into account only the trades that have been finished. Use ["true", "false"]
             unit: Optional[List[Enum]] ``FlowsMeasurementUnit``
             with_intra_country: Optional[bool] By default: withIntraCountry=false. Takes into account the trades within the selected country. Use ["true", "false"]
             with_intra_region: Optional[bool] By default: withIntraRegion=true. Takes into account the trades within the selected region. Use ["true", "false"]
             with_forecast: Optional[bool] By default: withForecast=true. Include trades predicted by our in-house model when set to "true". Use ["true", "false"]
             with_freight_view: bool By default: with_freight_view=False. Provides access to the entire fleet's trades, irrespective of your current cargo subscription. Only available via Freight subscription.
             vessel_types: Optional[List[Enum]] = Vessel types, pick types based on platform.
             vessel_types_alt: Optional[List[Enum]] = For oil vessel types, only available for Liquids
             with_product_estimation: bool By default withProductEstimation=False. Activate the product estimation feature. When set to true, the estimated product will overwrite the existing product information.
             snapshot_date: Optional[date] Flows snapshot date, If not provided, will return the live results (YYYY-MM-DD)
             
        Examples:

            >>> from datetime import date
            ... from kpler.sdk.resources.flows import Flows
            ... from kpler.sdk import FlowsDirection, FlowsSplit, FlowsPeriod, FlowsMeasurementUnit
            ... flows_client = Flows(config)
            ... flows_client.get(
            ...     flow_direction=[FlowsDirection.Export],
            ...     split=[FlowsSplit.OriginCountries],
            ...     granularity=[FlowsPeriod.Daily],
            ...     unit=[FlowsMeasurementUnit.T],
            ...     start_date=date(2020,9,1),
            ...     end_date=date(2020,10,1)
            ... )

            .. csv-table::
                :header:  "","Belgium","Singapore Republic","Equatorial Guinea","Trinidad and Tobago","Peru","Qatar","Netherlands","Cameroon","France","..."

                "2020-09-01","0.00","0.00","0.00","31503.98","0.00","444830.12","0.0","0.00","0.00","..."
                "2020-09-02","0.00","0.00","0.00","29470.38","0.00","127286.51","0.0","0.00","0.00","..."
                "2020-09-03","0.00","0.00","0.00","0.00","0.00","217991.87","0.0","0.00","0.00","..."
                "2020-09-04","0.00","0.00","0.00","0.00","0.00","279156.75","0.0","0.00","75441.15","..."
                "2020-09-05","0.00","450.00","0.00","55242.96","77166.98","96567.48","3038.4","0.00","0.00","..."
                "2020-09-06","0.00","0.00","0.00","66487.50","0.00","281436.69","71762.4","34352.10","0.00",".."
                "..","..","..","..","..","..","..","..","..","..",".."
        """

        query_parameters = {
            "flowDirection": process_enum_parameters(flow_direction),
            "split": process_enum_parameters(split),
            "granularity": process_enum_parameters(granularity),
            "startDate": process_date_parameter(start_date),
            "endDate": process_date_parameter(end_date),
            "fromInstallations": process_list_parameter(from_installations),
            "toInstallations": process_list_parameter(to_installations),
            "fromZones": process_list_parameter(from_zones),
            "toZones": process_list_parameter(to_zones),
            "products": process_list_parameter(products),
            "onlyRealized": process_bool_parameter(only_realized),
            "unit": process_enum_parameters(unit),
            "withIntraCountry": process_bool_parameter(with_intra_country),
            "withIntraRegion": process_bool_parameter(with_intra_region),
            "withForecast": process_bool_parameter(with_forecast),
            "withFreightView": process_bool_parameter(with_freight_view),
            "vesselTypes": process_enum_parameters(vessel_types, to_lower_case=False),
            "vesselTypesAlt": process_enum_parameters(vessel_types_alt, to_lower_case=False),
            "withProductEstimation": process_bool_parameter(with_product_estimation),
            "snapshotDate": process_flow_snapshot_parameter(snapshot_date), 
        }
        return self._get_dataframe(self.RESOURCE_NAME, query_parameters)



from datetime import date
from enum import Enum
from typing import Any, Dict, List, Optional

from pandas import DataFrame

from kpler.sdk import Platform
from kpler.sdk.client import KplerClient
from kpler.sdk.configuration import Configuration
from kpler.sdk.exceptions import WithProductEstimationPlatformException
from kpler.sdk.helpers import (
    process_bool_parameter,
    process_date_parameter,
    process_enum_parameters,
    process_list_parameter,
)


class Trades(KplerClient):
    """
    The ``Trades`` query returns the volumes from one point of interest to another (installation/zone) on a cargo-by-cargo basis.
    """

    RESOURCE_NAME = "trades"

    AVAILABLE_PLATFORMS = [Platform.Dry, Platform.Liquids, Platform.LNG, Platform.LPG]

    def __init__(self, configuration: Configuration, column_ids: bool = True, log_level=None):
        super().__init__(configuration, self.AVAILABLE_PLATFORMS, column_ids, log_level)

    def get_columns(self) -> DataFrame:
        """
        This endpoint returns a recent and updated list of all columns available for the endpoint trades.

        Examples:
            >>> from kpler.sdk.resources.trades import Trades
            ...   trades_client = Trades(config)
            ...   trades_client.get_columns()

            .. csv-table::
                :header: "id","name","description","deprecated","type"

                "vessel_name","Vessel","Name of the vessel","False","string"
                "start","Date (origin)","Departure date of the vessel","False","datetime yyyy-MM-dd HH:mm"
                "origin_location_name","Origin","Origin location of the cargo","False","string"
                "origin_eta_source","Eta source (origin)","Source of the Estimated Time of Arrival to the Installation of Origin information (Port, Analyst, etc.)","False","string"
                "cargo_origin_cubic_meters","Volume (origin m3)","None","False","long"
                "...","...","...","...","..."
        """
        return self._get_columns_for_resource(self.RESOURCE_NAME)

    def get(
        self,
        size: Optional[int] = None,
        vessels: Optional[List[str]] = None,
        from_installations: Optional[List[str]] = None,
        to_installations: Optional[List[str]] = None,
        from_zones: Optional[List[str]] = None,
        to_zones: Optional[List[str]] = None,
        buyers: Optional[List[str]] = None,
        sellers: Optional[List[str]] = None,
        products: Optional[List[str]] = None,
        start_date: Optional[date] = None,
        end_date: Optional[date] = None,
        trade_status: Optional[List[Enum]] = None,
        with_forecast: Optional[bool] = None,
        with_intra_country: Optional[bool] = None,
        with_intra_region: Optional[bool] = None,
        columns: Optional[List[str]] = None,
        with_freight_view: Optional[bool] = False,
        with_product_estimation: Optional[bool] = False,
    ) -> DataFrame:
        """

        Args:
            size: Optional[int] Maximum number of trades returned
            vessels: Optional[List[str]] Names/IMO's of vessels
            from_installations: Optional[List[str]] Names of the origin installations
            to_installations: Optional[List[str]] Names of the destination installations (terminal/refinery)
            from_zones: Optional[List[str]] Names of the origin zones (port/region/country/continent)
            to_zones: Optional[List[str]] Names of the destination zones (port/region/country/continent)
            buyers: Optional[List[str]] Buyers of the cargo
            sellers: Optional[List[str]] Sellers of the cargo
            products: Optional[List[str]] Names of products
            start_date: Optional[date] Start of the period
            end_date: Optional[date] End of the period
            trade_status: Optional[List[Enum]] ``TradesStatus`` Return only trades of a particular status. By default value is scheduled.
            with_forecast: Optional[bool] By default: withForecast=true. Include trades predicted by our in-house model when set to "true". Use ["true", "false"]
            with_intra_country: Optional[bool] By default: withIntraCountry=true. Takes into account the trades within the selected country. Use ["true", "false"]
            with_intra_region: Optional[bool] By default: withIntraRegion=true. Takes into account the trades within the selected region. Use ["true", "false"]
            columns: Optional[List[str]] Retrieve all available columns when set to "all"
            with_freight_view: bool By default: with_freight_view=False. Provides access to the entire fleet's trades, irrespective of your current cargo subscription. Only available via Freight subscription.
            with_product_estimation: bool By default withProductEstimation=False. Activate the product estimation feature. When set to true, the estimated product will overwrite the existing product information.
        Examples:
            >>> from datetime import date, timedelta
            ... from kpler.sdk.resources.trades import Trades
            ... trades_client = Trades(config)
            ... trades_client.get(
            ... to_zones=["United States"],
            ... products=["crude"],
            ... with_forecast=False,
            ... with_intra_country=True,
            ... start_date=date.today() - timedelta(days=7),
            ... columns=[
            ...     "vessel_name",
            ...     "closest_ancestor_product",
            ...     "closest_ancestor_grade",
            ...     "start",
            ...     "end",
            ...     "origin_location_name",
            ...     "destination_location_name"
            ... ]
            ... )

            .. csv-table::
                :header: "vessel_name","closest_ancestor_product","closest_ancestor_grade","start","end","origin_location_name","destination_location_name"

                "Eco Bel Air","crude","Basrah","2020-11-01 14:20:00","2020-12-14 01:02:00","Al Basrah","PADD 5"
                "Stella","crude","NaN","2020-10-30 09:48:00","2020-12-09 21:30:00","Angra dos Reis","PADD 5"
                "Cap Charles","crude","NaN","2020-10-21 05:47:00","2020-11-20 23:15:00","Angra dos Reis","Long Beach"
                "Sebarok Spirit","crude","Maya","2020-10-21 01:00:00","2020-10-24 06:49:00","Yuum Kak Naab FPSO","Houston"
                "Montreal Spirit","crude","NaN","2020-10-20 13:30:00","2020-11-24 01:18:00","Sao Sebastiao","Cherry Point"
                "Washington","crude","ANS","2020-10-19 14:48:00","2020-10-27 00:00:00","Valdez","San Francisco"
                "...","...","...","...","...","...","..."
        """

        self.with_product_estimation = with_product_estimation

        query_parameters: Dict[str, Optional[Any]] = {
            "size": size,
            "vessels": process_list_parameter(vessels),
            "fromInstallations": process_list_parameter(from_installations),
            "toInstallations": process_list_parameter(to_installations),
            "fromZones": process_list_parameter(from_zones),
            "toZones": process_list_parameter(to_zones),
            "buyers": process_list_parameter(buyers),
            "sellers": process_list_parameter(sellers),
            "products": process_list_parameter(products),
            "startDate": process_date_parameter(start_date),
            "endDate": process_date_parameter(end_date),
            "tradeStatus": process_enum_parameters(trade_status),
            "withForecast": process_bool_parameter(with_forecast),
            "withIntraCountry": process_bool_parameter(with_intra_country),
            "withIntraRegion": process_bool_parameter(with_intra_region),
            "columns": process_list_parameter(columns),
            "withFreightView": process_bool_parameter(with_freight_view),
            "withProductEstimation": process_bool_parameter(with_product_estimation),
        }
        return self._get_dataframe(self.RESOURCE_NAME, query_parameters)

    def validate(self):
        if self.with_product_estimation and self.configuration.platform != Platform.Liquids:
            raise WithProductEstimationPlatformException()



from setuptools import find_namespace_packages, setup


with open("README.md") as fh:
    long_description = fh.read()

setup(
    name="kpler.sdk",
    description="A Python wrapper around the Kpler client API",
    use_scm_version=True,
    setup_requires=["setuptools_scm"],
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="http://github.com/kpler/python-sdk",
    author="Kpler",
    author_email="engineering@kpler.com",
    license="Apache License, Version 2.0",
    packages=find_namespace_packages(
        where="src",
        include=["kpler*"],
    ),
    package_dir={"": "src"},
    install_requires=[
        "pandas>=1.0.0,<=1.5.3",
        "requests>=2.20.0,<=2.31.0",
        "numpy<2",
    ],
    zip_safe=False,
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
    extras_require={
        "doc": [
            "Sphinx==5.0.0",
            "sphinx_rtd_theme==1.0.0",
            "sphinx-autodoc-typehints==1.12.0",
        ],
        "test": [
            "pytest==6.2.5",
            "python-dateutil==2.8.1",
            "pytest-rerunfailures==11.1.2",
            "black==18.9b0",
            "mypy-extensions==1.0.0",
            "mypy==1.4.1",
            "pre-commit==2.6.0",
        ],
        "publish": [
            "twine==4.0.2",
            "urllib3>=1.21.1,<1.27",
            "importlib-metadata<8.0.0"
        ],
    },
)



from datetime import date
from typing import List, Optional

from pandas import DataFrame

from kpler.sdk import Platform
from kpler.sdk.client import KplerClient
from kpler.sdk.configuration import Configuration
from kpler.sdk.helpers import (
    process_date_parameter,
    process_enum_parameters,
    process_list_parameter,
)


class Diversions(KplerClient):

    """
    The ``Diversions`` endpoint allows users to extract a list of historical & current diversions for all LNG vessels
    going back to 2019.
    """

    RESOURCE_NAME = "diversions"

    AVAILABLE_PLATFORMS = [Platform.LNG]

    def __init__(self, configuration: Configuration, column_ids: bool = True, log_level=None):
        super().__init__(configuration, self.AVAILABLE_PLATFORMS, column_ids, log_level)

    def get_columns(self) -> DataFrame:
        """
        This endpoint returns a recent and updated list of all columns available for the endpoint diversions.

        Examples:
            >>> from kpler.sdk.resources.diversions import Diversions
            ... diversions_client = Diversions(config)
            ... diversions_client.get_columns()

            .. csv-table::
                :header: "id","name","description","deprecated","type"

                "vessel_name","Vessel","None","False","string"
                "diversion_date","Diversion date","None","False","datetime yyyy-MM-dd HH:mm"
                "origin_diversion_location_name","Origin","None","False","string"
                "origin_diversion_date","Origin Date","None","False","datetime yyyy-MM-dd HH:mm"
                "diverted_from_location_name","Diverted from","None","False","string"
                "new_destination_location_name","New Destination","None","False","string"
                "new_destination_date","New Destination Date","None","False","datetime yyyy-MM-dd HH:mm"
                "vessel_state","Vessel State","None","False","string"
                "charterer_name","Charterer","None","False","string"
                "...","...","...","...","..."
        """

        return self._get_columns_for_resource(self.RESOURCE_NAME)

    def get(
        self,
        size: Optional[int] = None,
        start_date: Optional[date] = None,
        end_date: Optional[date] = None,
        from_zones: Optional[List[str]] = None,
        to_zones: Optional[List[str]] = None,
        cancelled_zones: Optional[List[str]] = None,
        from_installations: Optional[List[str]] = None,
        to_installations: Optional[List[str]] = None,
        cancelled_installations: Optional[List[str]] = None,
        vessels: Optional[List[str]] = None,
        charterers: Optional[List[str]] = None,
        vessel_states: Optional[List[str]] = None,
        columns: Optional[List[str]] = None,
    ):
        """

        Args:
            size: Optional[int] Maximum number of diversions returned
            start_date: Optional[date] Start of the period (YYYY-MM-DD)
            end_date: Optional[date] End of the period (YYYY-MM-DD)
            from_zones: Optional[List[str]] Names of origin zones ["port", "region", "country", "continent"]
            to_zones: Optional[List[str]] Names of destination zones
            cancelled_zones: Optional[List[str]] Names of diverted zones
            from_installations: Optional[List[str]] Names of origin installations
            to_installations: Optional[List[str]] Names of destination installations
            cancelled_installations: Optional[List[str]] Names of diverted installations
            vessels: Optional[List[str]] Names or IMOs of vessels
            charterers: Optional[List[str]] Names or charterers
            vessel_states: Optional[List[Enum]] = ``DiversionsVesselState``
            columns: Optional[List[str]] Retrieve all available columns when set to "all"

        Examples:
            >>> from kpler.sdk.resources.diversions import Diversions
            ... from kpler.sdk import DiversionsVesselState
            ... diversions_client = Diversions(config)
            ... diversions_client.get(
            ...     from_installations=["Sabine Pass"],
            ...     to_zones=["United States"],
            ...     size=10,
            ...     vessel_states=[DiversionsVesselState.Loaded]
            ... )

            .. csv-table::
                :header: "vessel_name","diversion_date","origin_diversion_location_name","origin_diversion_date","diverted_from_location_name","new_destination_location_name","new_destination_date","vessel_state","charterer_name"

                "Hispania Spirit","2020-12-23 15:31:00","Sabine Pass","2020-12-17 08:57:00","Dragon","Rio","2021-01-11 11:27:00","Loaded","Shell"
                "Gaslog Skagen","2020-11-30 00:00:00","Sabine Pass","2020-11-25 16:31:00","Dragon","Bahia","2020-12-06 16:01:00","Loaded","RWE"
                "Maran Gas Ulysses","2020-03-05 00:00:00","Sabine Pass","2020-03-01 07:02:00","MED Sea","Quintero","2020-06-05 16:42:00","Loaded","Shell"
        """

        query_parameters = {
            "size": size,
            "startDate": process_date_parameter(start_date),
            "endDate": process_date_parameter(end_date),
            "fromZones": process_list_parameter(from_zones),
            "toZones": process_list_parameter(to_zones),
            "cancelledZones": process_list_parameter(cancelled_zones),
            "fromInstallations": process_list_parameter(from_installations),
            "toInstallations": process_list_parameter(to_installations),
            "cancelledInstallations": process_list_parameter(cancelled_installations),
            "vessels": process_list_parameter(vessels),
            "charterers": process_list_parameter(charterers),
            "vessel_states": process_enum_parameters(vessel_states),
            "columns": process_list_parameter(columns),
        }
        return self._get_dataframe(self.RESOURCE_NAME, query_parameters)



# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
# import os
# import sys
# sys.path.insert(0, os.path.abspath('.'))

from pkg_resources import get_distribution


__version__ = get_distribution("kpler.sdk").version

# The short X.Y version.
version = __version__
# The full version, including alpha/beta/rc tags.
release = version


# -- Project information -----------------------------------------------------

project = f"Kpler python SDK {version}"
copyright = "2020, Kpler"
author = "Kpler"

# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    "sphinx.ext.autodoc",
    "sphinx_autodoc_typehints",
    "sphinx.ext.coverage",
    "sphinx.ext.doctest",
    "sphinx.ext.graphviz",
    "sphinx.ext.intersphinx",
    "sphinx.ext.mathjax",
    "sphinx.ext.napoleon",
    "sphinx.ext.todo",
    "sphinx.ext.viewcode",
    "sphinx.ext.autosectionlabel",
]

# Add any paths that contain templates here, relative to this directory.
templates_path = ["_templates"]

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = []  # type: ignore

# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
# html_theme = 'alabaster'
html_theme = "sphinx_rtd_theme"
html_theme_options = {
    "display_version": True,
    # Toc options
    "collapse_navigation": True,
    "sticky_navigation": True,
    "navigation_depth": 4,
    "includehidden": True,
    "titles_only": False,
}


# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ["_static"]

# The name of an image file (relative to this directory) to place at the top
# of the sidebar.
# html_logo = 'logo.png'  # Orange on transparent background
html_logo = "_static/logo.png"

# The name of an image file (within the static path) to use as favicon of the
# docs.  This file should be a Windows icon file (.ico) being 16x16 or 32x32
# pixels large.
html_favicon = "_static/favicon.ico"


def setup(app):
    app.add_css_file("css/custom.css")



import json
import logging
from typing import Mapping, Optional, Tuple, Union

from pkg_resources import get_distribution
import requests
from requests.auth import HTTPBasicAuth

from kpler.sdk.enums import Platform


class Configuration:
    def __init__(
        self,
        platform: Platform,
        email: str,
        password: str,
        proxies: Optional[Mapping[str, str]] = None,
        certificates: Optional[Union[Tuple[str, str], str]] = None,
        verify: Optional[bool] = True,
        log_level=None,
        forced_base_url: Optional[str] = None,
    ):

        self.proxies = proxies
        self.certificates = certificates
        self.verify = verify
        self.forced_base_url = forced_base_url

        self.platform = platform

        self.email = email
        self.password = password
        self.auth_validated = False

        logging.basicConfig(format="%(asctime)s [%(levelname)s] %(message)s")
        self.logger = logging.getLogger("kpler")
        self.logger.setLevel(log_level or logging.ERROR)

        self._authenticate()
        if self.auth_validated:
            self._check_new_version()

    @property
    def base_url(self) -> str:
        return self.forced_base_url if self.forced_base_url else str(self.platform.value)

    def _authenticate(self) -> None:
        if self.auth_validated:
            return

        response = requests.get(
            f"{self.base_url}/trades/columns",
            auth=HTTPBasicAuth(self.email, self.password),
            proxies=self.proxies,
            cert=self.certificates,
            verify=self.verify,
        )
        email_platform_str = f"'{self.email}' on '{self.platform}'"
        if response.status_code == 200:
            self.auth_validated = True
            self.logger.info(f"Credentials validated for {email_platform_str}")
        else:
            self.auth_validated = False
            self.logger.error(
                f"Unable to validate your credentials for {email_platform_str}: {response.content}"  # type: ignore
            )

    def _change_platform(self, platform: Platform) -> None:
        self.platform = platform
        self._authenticate()

    def _check_new_version(self) -> None:
        version = self.get_clean_version(get_distribution("kpler.sdk").version)
        response = requests.get(
            f"{self.base_url}/clients/python_sdk/versions?afterVersion={version}&size=1",
            auth=HTTPBasicAuth(self.email, self.password),
            proxies=self.proxies,
            cert=self.certificates,
            verify=self.verify,
        )
        if response.status_code != 200:
            self.logger.error(
                f"Unable to validate your credentials for {self.email}: {response.content}"
            )
        else:
            json_data = json.loads(response.text)
            if isinstance(json_data, list) and len(json_data) > 0:
                self.logger.info(
                    f"A new versions {json_data[0]['clientVersion']} is available, please upgrade the Kpler SDK."
                )

    def get_clean_version(self, version) -> str:
        return ".".join(version.split(".")[:3])



from datetime import date
from enum import Enum
from typing import List, Optional

from pandas import DataFrame

from kpler.sdk import Platform
from kpler.sdk.client import KplerClient
from kpler.sdk.configuration import Configuration
from kpler.sdk.helpers import (
    process_date_parameter,
    process_enum_parameters,
    process_list_parameter,
)


class Contracts(KplerClient):

    """
    The ``Contracts`` endpoint allows you to extract a list of SPAs, TUAs, LTAs, and Tenders for all LNG players, installations and zones.
    """

    RESOURCE_NAME = "contracts"

    AVAILABLE_PLATFORMS = [Platform.LNG]

    def __init__(self, configuration: Configuration, column_ids: bool = True, log_level=None):
        super().__init__(configuration, self.AVAILABLE_PLATFORMS, column_ids, log_level)

    def get_columns(self) -> DataFrame:
        """

        Examples:
           >>> from kpler.sdk.resources.contracts import Contracts
           ... contracts_client = Contracts(config)
           ... contracts_client.get_columns()

            .. csv-table::
                :header: "id","name","description","deprecated","type"

                "type","Type","Type of contracts (SPA, LTA, TUA, Tender)","False","string"
                "seller","Seller","Company name of the seller","False","string"
                "buyer","Buyer","Company name of the buyer","False","string"
                "capacity","Capacity","None","False","float"
                "slots","Slots","None","False","integer"
                "...","...","...","...","..."
        """

        return self._get_columns_for_resource(self.RESOURCE_NAME)

    def get(
        self,
        columns: Optional[List[str]] = None,
        size: Optional[int] = None,
        start_date: Optional[date] = None,
        end_date: Optional[date] = None,
        players: Optional[List[str]] = None,
        types: Optional[List[Enum]] = None,
        installations: Optional[List[str]] = None,
        zones: Optional[List[str]] = None,
    ):
        """
        This endpoint returns a list of contracts for the given arguments.

        Args:
            size:  Optional[int] Maximum number of contracts returned
            start_date:  Optional[date] Start of the period (YYYY-MM-DD)
            end_date:  Optional[date] End of the period (YYYY-MM-DD)
            players: Optional[str] Represents the buyers/sellers on the contracts
            types: Optional[List[Enum]] ``ContractsTypes``
            installations: Optional[List[str]] Names of installations
            zones: Optional[List[str]] Names of countries/geographical zones

        Examples:
            >>> from datetime import date
            ... from kpler.sdk.resources.contracts import Contracts
            ... from kpler.sdk import ContractsTypes
            ... contracts_client = Contracts(config)
            ... contracts_client.get(
            ...     size=10,
            ...     start_date=date(2018,1,1),
            ...     end_date=date(2020,10,1),
            ...     installations=["Dahej"],
            ...     zones=["India"],
            ...     players=["Shell"],
            ...     types=[ContractsTypes.Tender]
            ... )

            .. csv-table::
                :header: "type","seller","buyer","capacity","slots","delivery","start","end","installations origin","installations destination","origin zone","..."

                "Tender","Shell","IOC","NaN","1","DES","2019-08-15 00:00","2019-08-31 00:00","NaN","Dahej, Ennore LNG","World",".."
                "Tender","Shell","BPCL","NaN","1","DES","2018-08-20 00:00","2018-09-15 00:00","NaN","Dahej, Hazira","World",".."
                "Tender","Shell","IOC","NaN","1","DES","2018-05-01 00:00","2018-05-31 00:00","NaN","Dahej","World",".."
                "Tender","Shell","BPCL","NaN","1","DES","2018-05-01 00:00","2018-05-31 00:00","NaN","Dahej","World",".."
                "Tender","Shell","GAIL","NaN","2","DES","2018-04-10 00:00","2018-05-05 00:00","NaN","Dahej, Dabhol","World",".."
                "Tender","Shell","GAIL","NaN","1","DES","2018-02-15 00:00","2018-02-28 00:00","NaN","Dahej","World",".."
                "Tender","Shell","BPCL","NaN","1","DES","2018-01-19 00:00","2018-01-23 00:00","NaN","Dahej","World",".."
        """

        query_parameters = {
            "columns": process_list_parameter(columns),
            "size": size,
            "startDate": process_date_parameter(start_date),
            "endDate": process_date_parameter(end_date),
            "players": process_list_parameter(players),
            "types": process_enum_parameters(types),
            "installations": process_list_parameter(installations),
            "zones": process_list_parameter(zones),
        }
        return self._get_dataframe(self.RESOURCE_NAME, query_parameters)



from datetime import date
from typing import List, Optional

from pandas import DataFrame

from kpler.sdk import Platform
from kpler.sdk.client import KplerClient
from kpler.sdk.configuration import Configuration
from kpler.sdk.helpers import process_bool_parameter, process_date_parameter, process_list_parameter


class PortCalls(KplerClient):
    """
    The ``PortCalls`` query returns the cargo-by-cargo details for loadings/discharges taking place in a point of
    interest (installation/zone).
    """

    RESOURCE_NAME = "port-calls"

    AVAILABLE_PLATFORMS = [Platform.Dry, Platform.Liquids, Platform.LNG, Platform.LPG]

    def __init__(self, configuration: Configuration, column_ids: bool = True, log_level=None):
        super().__init__(configuration, self.AVAILABLE_PLATFORMS, column_ids, log_level)

    def get_columns(self) -> DataFrame:
        """
        This endpoint returns a recent and updated list of all columns available for the endpoint port-calls.

        Examples:
            >>> from kpler.sdk.resources.port_calls import PortCalls
            ... port_calls_client = PortCalls(config)
            ... port_calls_client.get_columns()

            .. csv-table::
                :header: "id","name","description","deprecated","type"

                "vessel_name","Vessel","Name of the vessel","False","string"
                "installation_name","Installation","Name of the installation","False","string"
                "eta","ETA","Estimated date and time of arrival at next destination","False","datetime yyyy-MM-dd HH:mm"
                "start","Start","Date and Time when the vessel started berthing at the terminal","False","datetime yyyy-MM-dd HH:mm"
                "end","End","Date and Time of vessel departure from the terminal","False","datetime yyyy-MM-dd HH:mm"
                "...","...","...","...","..."
        """
        return self._get_columns_for_resource(self.RESOURCE_NAME)

    def get(
        self,
        size: Optional[int] = None,
        vessels: Optional[List[str]] = None,
        installations: Optional[List[str]] = None,
        zones: Optional[List[str]] = None,
        products: Optional[List[str]] = None,
        start_date: Optional[date] = None,
        end_date: Optional[date] = None,
        with_forecast: Optional[bool] = None,
        columns: Optional[List[str]] = None,
        with_freight_view: bool = False,
    ) -> DataFrame:
        """

        Args:
             size: Optional[int] Maximum number of portcalls
             vessels: Optional[List[str]] Names/IMOs of vessels
             installations: Optional[List[str]] Names of installations
             zones: Optional[List[str]] Names of zones ["port", "region", "country", "continent"]
             products: Optional[List[str]] Names of products
             start_date: Optional[date] Start of the period (YYYY-MM-DD)
             end_date: Optional[date] End of the period (YYYY-MM-DD)
             with_forecast: Optional[bool] By default: withForecast=true. Include trades predicted by our in-house model when set to "true". Use ["true", "false"]
             columns: Optional[List[str]] Retrieve all available columns when set to "all"
             with_freight_view: bool By default: with_freight_view=False. Provides access to the entire fleet's trades, irrespective of your current cargo subscription. Only available via Freight subscription.

        Examples:
            >>> from kpler.sdk.resources.port_calls import PortCalls
            ... port_calls_client = PortCalls(config)
            ... port_calls_client.get(
            ...     start_date=date.today() - timedelta(days=7),
            ...     zones=["United States"],
            ...     installations=["Everglades"],
            ...     columns=["vessel_name","installation_name","eta","start","end","flow_quantity_cubic_meters"]
            ... )

            .. csv-table::
                :header: "vessel_name","installation_name","eta","start","end","flow_quantity_cubic_meters"

                "Palmetto State","Everglade","2020-12-11 21:00:00","NaT","NaT","-29681.0"
                "Golden State","Everglade","2020-12-10 17:00:00","NaT","NaT","-53477.0"
                "Overseas Houston","Everglade","2020-12-09 11:00:00","NaT","NaT","-51560.0"
                "Garden State","Everglade","2020-12-09 02:00:00","NaT","NaT","-54045.0"
                "American Liberty","Everglade","2020-12-08 22:00:00","NaT","NaT","-54736.0"
                "Louisiana","Everglade","2020-12-06 16:00:00","2020-12-06 18:01:00","2020-12-07 03:05:00","-9445.0"
                "California Voyager","Everglade","2020-12-04 03:00:00","2020-12-04 03:28:00","2020-12-05 11:51:00","-37295.0"
                "Brenton Reef","Everglade","2020-12-02 20:30:00","2020-12-02 21:56:00","2020-12-04 02:00:00","-50339.0"
                "...","...","...","...","...","..."
        """

        query_parameters = {
            "size": size,
            "vessels": process_list_parameter(vessels),
            "installations": process_list_parameter(installations),
            "zones": process_list_parameter(zones),
            "products": process_list_parameter(products),
            "startDate": process_date_parameter(start_date),
            "endDate": process_date_parameter(end_date),
            "withForecast": process_bool_parameter(with_forecast),
            "columns": process_list_parameter(columns),
            "withFreightView": process_bool_parameter(with_freight_view),
        }
        return self._get_dataframe(self.RESOURCE_NAME, query_parameters)



from typing import List, Optional

from pandas import DataFrame

from kpler.sdk import Platform
from kpler.sdk.client import KplerClient
from kpler.sdk.configuration import Configuration
from kpler.sdk.helpers import process_list_parameter


class Installations(KplerClient):
    """
    The ``Installations`` endpoint allows to perform full-text search on installations,
    in order to find names used in Kpler referential.
    """

    RESOURCE_NAME = "installations"

    AVAILABLE_PLATFORMS = [Platform.LNG]

    def __init__(self, configuration: Configuration, column_ids: bool = True, log_level=None):
        super().__init__(configuration, self.AVAILABLE_PLATFORMS, column_ids, log_level)

    def get_columns(self) -> DataFrame:
        """
        This endpoint returns a recent and updated list of all columns available for the endpoint installations.

        Examples:
            >>> from kpler.sdk.resources.installations import Installations
            ... installations_client = Installations(config)
            ... installations_client.get_columns()

            .. csv-table::
                :header: "id","name","description","deprecated","type"

                "continent","Continent","The installation's continent","False","string"
                "subcontinent","Subcontinent","The installation's subcontinent","False","string"
                "country","Country","The installation's country","False","string"
                "port","Port","The installation's port","False","string"
                "installation","Installation","The Installation's name","False","string"
                "...","...","...","...","..."
        """
        return self._get_columns_for_resource(self.RESOURCE_NAME)

    def get(
        self,
        columns: Optional[List[str]] = None,
        continent: Optional[str] = None,
        country: Optional[str] = None,
        port: Optional[str] = None,
        installation: Optional[str] = None,
        continent_id: Optional[int] = None,
        country_id: Optional[int] = None,
        port_id: Optional[int] = None,
        installation_id: Optional[int] = None,
        type: Optional[str] = None,
        owner: Optional[str] = None,
    ) -> DataFrame:
        """

        Args:
            continent:  Optional[str] The installation's continent name
            country:  Optional[str] The installation's country name
            port:  Optional[str] The installation's port name
            installation:  Optional[str] The installation's name
            continent_id:  Optional[int] The installation's continent ID in the Kpler system
            country_id:  Optional[int] The installation's country ID in the Kpler system
            port_id:  Optional[int] The installation's port ID in the Kpler system
            installation_id:  Optional[int] The installation's ID in the Kpler system
            type:  Optional[str] The installation's type, Import/Export
            owner:  Optional[str] The installation's owner
            columns: Optional[List[str]] Retrieve all available columns when set to "all"

        Examples:
            >>> from kpler.sdk.resources.installations import Installations
            ... installations_client = Installations(config)
            ... installations_client.get(columns=["installation","installation_type", "status","lng_storage_capacity"])

            .. csv-table::
                :header: "installation","installation_type", "status","lng_storage_capacity"

                "Fos Tonkin","Import","Active","80000"
                "Fos Cavaou","Import","Active","330000"
                "Freeport (Reg.)","Import","Decommissioned","480000"
                "Delfin FLNG","Export","Approved",""
                "Freeport","Export","Active","480000"
                "...","...","...","..."
        """
        query_parameters = {
            "continent": process_list_parameter(continent),
            "country": process_list_parameter(country),
            "port": process_list_parameter(port),
            "installation": process_list_parameter(installation),
            "continentId": process_list_parameter(continent_id),
            "countryId": process_list_parameter(country_id),
            "portId": process_list_parameter(port_id),
            "installationId": process_list_parameter(installation_id),
            "type": process_list_parameter(type),
            "owner": process_list_parameter(owner),
            "columns": process_list_parameter(columns),
        }
        return self._get_dataframe(self.RESOURCE_NAME, query_parameters)

    def search(self, q: str) -> DataFrame:
        """

        Args:
            q: str Argument to search by in installation names

        Examples:
            >>> from kpler.sdk.resources.installations import Installations
            ... installations_client=Installations(config)
            ... installations_client.search("abidjan")

            .. csv-table::
                :header: "installations"

                "SIR Abidjan"
                "Abidjan Terminal"

        """
        query_parameters = {"q": q, "resources": self.RESOURCE_NAME}
        return self._search(query_parameters)



from typing import List, Optional

from pandas import DataFrame

from kpler.sdk import Platform
from kpler.sdk.client import KplerClient
from kpler.sdk.configuration import Configuration
from kpler.sdk.helpers import process_list_parameter


class Products(KplerClient):
    """
    The ``Products`` endpoint allows to perform full-text search on products,
    in order to find names used in Kpler referential.
    """

    RESOURCE_NAME = "products"

    AVAILABLE_PLATFORMS = [Platform.Dry, Platform.Liquids, Platform.LNG, Platform.LPG]

    def __init__(self, configuration: Configuration, column_ids: bool = True, log_level=None):
        super().__init__(configuration, self.AVAILABLE_PLATFORMS, column_ids, log_level)

    def get_columns(self) -> DataFrame:
        """
        This endpoint returns a recent and updated list of all columns available for the endpoint products.

        Examples:
            >>> from kpler.sdk.resources.products import Products
            ... products_client = Products(config)
            ... products_client.get_columns()

            .. csv-table::
                :header: "id","name","description","deprecated","type"

                "id","Id (Product)","Identifier in the database of Kpler","False","long"
                "name","Name","Name of the product","False","string"
                "product_type","Type (Product)","Product Type","False","string"
                "family_name","Family","closest Family to the product","False","string"
                "family_id","Family Id","Id of the closest Family to the product","False","long"
                "...","...","...","...","..."
        """
        return self._get_columns_for_resource(self.RESOURCE_NAME)

    def get(
        self,
        columns: Optional[List[str]] = None,
        size: Optional[int] = None,
        ancestor_family_ids: Optional[List[int]] = None,
        ancestor_family_names: Optional[List[str]] = None,
        ancestor_group_ids: Optional[List[int]] = None,
        ancestor_group_names: Optional[List[str]] = None,
        ancestor_product_ids: Optional[List[int]] = None,
        ancestor_product_names: Optional[List[str]] = None,
        ancestor_grade_ids: Optional[List[int]] = None,
        ancestor_grade_names: Optional[List[str]] = None,
        products: Optional[List[str]] = None,
        product_ids: Optional[List[int]] = None,
    ) -> DataFrame:
        """

        Args:
            size:  Optional[int]
            ancestor_family_ids:  Optional[List[int]] IDs of a product Family. Can be separated by a coma
            ancestor_family_names:  Optional[List[str]] Default to the whole tree if not specified. Possible values are : Light Ends, NPC, DPP, Middle Distillate, Dirty
            ancestor_group_ids:  Optional[List[int]] IDs of a product Group. Can be separated by a coma
            ancestor_group_names:  Optional[List[str]] Default to the whole tree if not specified. Possible values are : Kero/Jet, Crude/Co, , Fuel Oils, etc...
            ancestor_product_ids:  Optional[List[int]] IDs of a product Product. Can be separated by a coma.
            ancestor_product_names:  Optional[List[str]] Default to the whole tree if not specified. Possible values are : Jet, Crude, Clean Condensate, etc ...
            ancestor_grade_ids:  Optional[List[int]] IDs of a product Grade. Can be separated by a coma.
            ancestor_grade_names:  Optional[List[str]] Default to the whole tree if not specified. Possible values are : RBOB, Solvent Naphtha, LSHO, 91 Gasoline,...
            products:  Optional[List[str]] Names of Products of all different tree levels (Family, Group, Product, Grade)
            product_ids: Optional[List[int]] Ids of possible products. Can be separated by a coma
            columns: Optional[List[str]] Retrieve all available columns when set to "all"

        Examples:
            >>> from kpler.sdk.resources.products import Products
            ... products_client = Products(config)
            ... products_client.get(ancestor_group_names=["Fuel Oils"], columns=["id", "family_name", "group_name", "product_name", "grade_name"])

            .. csv-table::
                :header: "id", "family_name", "group_name", "product_name", "grade_name"

                "2715","Dirty","Crude/Co","Crude","EOPL Heavy"
                "2716","Chem/Bio","Vegoils/Biofuels","Biofuels","SAF"
                "2717","Middle Distillates","Gasoil/Diesel","Diesel","Diesel B5"
                "2718","Dirty","Crude/Co","Crude","Suez Blend"
                "2728","Chem/Bio","Vegoils/Biofuels","Animal Fats",""
                "...","...","...","..."
        """
        query_parameters = {
            "size": process_list_parameter(size),
            "ancestorFamilyIds": process_list_parameter(ancestor_family_ids),
            "ancestorFamilyNames": process_list_parameter(ancestor_family_names),
            "ancestorGroupIds": process_list_parameter(ancestor_group_ids),
            "ancestorGroupNames": process_list_parameter(ancestor_group_names),
            "ancestorProductIds": process_list_parameter(ancestor_product_ids),
            "ancestorProductNames": process_list_parameter(ancestor_product_names),
            "ancestorGradeIds": process_list_parameter(ancestor_grade_ids),
            "ancestorGradeNames": process_list_parameter(ancestor_grade_names),
            "products": process_list_parameter(products),
            "productIds": process_list_parameter(product_ids),
            "columns": process_list_parameter(columns),
        }
        return self._get_dataframe(self.RESOURCE_NAME, query_parameters)

    def search(self, q: str) -> DataFrame:
        """

        Args:
            q: str Argument to search by in products names

        Examples:
            >>> from kpler.sdk.resources.products import Products
            ... products_client=Products(config)
            ... products_client.search("Arab")

            .. csv-table::
                :header: "products"

                "Arab"
                "Arab M"
                "Arab M Abu Safah"
                "Arab SLt."
                "Arab XLt."
                "Arab Hy."
                "Arab Lt."
        """
        query_parameters = {"q": q, "resources": self.RESOURCE_NAME}
        return self._search(query_parameters)
